/*
** net_api.c
**
** Made by mycroft (mycroft)
** <mycroft@minithins.net> on virgaria.minithins.net
**
** Started on  Wed Jan  1 16:23:03 2003 mycroft
** Last update Wed Jan  1 17:16:30 2003 mycroft
*/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <signal.h>

#include "net_api.h"

unsigned char signaux_en_attente = 0;

int net_init (void)
{
  if (access (FILE_NET_DST, F_OK) == 0)
    if (unlink (FILE_NET_DST) != 0)
    {
      fprintf (stderr, "Ne peut effacer %s\n.", FILE_NET_DST);
      exit (EXIT_FAILURE);
    }

  if (access (FILE_NET_SRC, F_OK) == 0)
    if (unlink (FILE_NET_SRC) != 0)
    {
      fprintf (stderr, "Ne peut effacer %s\n.", FILE_NET_SRC);
      exit (EXIT_FAILURE);
    }

  if (access (FILE_PHY, F_OK) == 0)
    if (unlink (FILE_PHY) != 0)
    {
      fprintf (stderr, "Ne peut effacer %s\n.", FILE_PHY);
      exit (EXIT_FAILURE);
    }

  return (0);
}

int net_close (void)
{
  unlink (FILE_PHY);
  unlink (FILE_NET_SRC);
  unlink (FILE_NET_DST);

  return (0);
}

/* Renvoit 0 si FILE_NET et FILE_PHY existent */
/* Pour le 'sender', qui ne pourra se lancer que quand le recepteur
   aura creer ces fichiers */

int net_ok (void)
{
  if (!access (FILE_PHY, F_OK) 
   && !access (FILE_NET_SRC, F_OK)
   && !access (FILE_NET_DST, F_OK))
    return (0);

  return (-1);
}

int Attendre (unsigned char signaux)
{
  unsigned char sig = 0;
  struct stat highwaytohell;

  fprintf (stderr, "attente du signal: %i\n", signaux);

  do
  {
    sleep (1);

    if (!sig && (signaux & SigEmis) == SigEmis)
    {
      /* On regarde si la taille du fichier est != de 0 */
      if (stat (FILE_NET_SRC, &highwaytohell) == -1)
      {
        /*perror ("stat net-source");
          */
/*        exit (EXIT_FAILURE); */
      }
      else
        if (highwaytohell.st_size != 0)
        {
          sig = SigEmis;
          signaux_en_attente |= SigEmis;
        }
    }

    if (!sig && (signaux & SigRecPhy) == SigRecPhy)
    {
      printf("data?\n");
      /* Pareil ci dessus */
      if (stat (FILE_PHY, &highwaytohell) == -1)
      {
        /*perror ("stat physique");*/
/*        exit (EXIT_FAILURE); */
      }
      else
        if (highwaytohell.st_size != 0)
        {
          sig = SigRecPhy;
          signaux_en_attente |= SigRecPhy;
        }
    }

    if (!sig && (signaux & SigRec) == SigRec)
    {
      if (stat (FILE_NET_DST, &highwaytohell) == -1)
      {
        /*perror ("stat");*/
        exit (EXIT_FAILURE);
      }
      else
        if (highwaytohell.st_size == 0)
        {
          sig = SigRec;
          signaux_en_attente |= SigRec;
        }
    }

    if (!sig && (signaux & SigTempo) == SigTempo)
    {
      if ( signaux_en_attente & SigTempo )
      {
        sig = SigTempo;
        signaux_en_attente |= SigTempo;
      }
    }
  }
  while (!sig);

  return (sig);
}

void Effacer (unsigned char signaux)
{
  int fd;
  fprintf (stderr, "effacement du signal: %i\n", signaux);

  if ((signaux & SigRecPhy) == SigRecPhy)
  {
    /* Vider le fichier.*/
    fd = open (FILE_PHY, O_RDWR | O_TRUNC);
    close (fd);
  }

  if ((signaux & SigRec) == SigRec)
  {
    /* Vider le fichier.*/
    fd = open (FILE_NET_DST, O_RDWR | O_TRUNC);
    close (fd);
  }

  if ((signaux & signaux_en_attente) == signaux)
    signaux_en_attente -= signaux;

  return;
}

int Signale (unsigned char signaux)
{
  if ((signaux_en_attente & signaux) == signaux)
    return 1;
  else
    return 0;
}

void RecvAlarm (int signal)
{
  if ((signaux_en_attente & SigTempo) == 0x00)
  {
    signaux_en_attente += SigTempo;
  }

  fprintf (stderr, "SigAlarm recieved.\n");
  return ;
}

void Tempo (unsigned int t)
{
  signal (SIGALRM, RecvAlarm);
  alarm (t);

  return ;
}

int Prendre_de_la_couche_reseau (
        TAddress *AS,
        TAddress *AD,
        Tampon *donnes)
{
  int fd;

  fd = open (FILE_NET_SRC, O_RDWR);
  read (fd, AS, 1);
  read (fd, AD, 1);
  read (fd, &(donnes->taille), 1);

  if (donnes->ptr_data)
    free (donnes->ptr_data);
  donnes->ptr_data = malloc (sizeof (unsigned char) * donnes->taille);

  read (fd, donnes->ptr_data, donnes->taille);

  close (fd);

  printf ("Vidage du fichier %s\n", FILE_NET_SRC);
  fd = open (FILE_NET_SRC, O_RDWR | O_TRUNC);
  close (fd);

  return (0);
}

unsigned char calcule_crc (unsigned char len, unsigned char *data)
{
  unsigned char crc = 0;
  int i;

  for (i = 0; i < len; i ++)
  {
    crc ^= *(data + i);
  }

  fprintf (stderr, "crc calcule: %i\n", crc);

  return (crc);
}

int Construire_trame (
        TAddress AS,
        TAddress AD,
        Tampon T,
        unsigned short num,
        TTrame *Trame)
{
  Trame->AS = AS;
  Trame->AD = AD; 

  Trame->TYPE = DATA;
  Trame->taille = T.taille;
  /* on ne se foule pas de toutes facons ... */

  Trame->num = num;
  Trame->ptr_data = T.ptr_data;
  Trame->crc = calcule_crc (Trame->taille, Trame->ptr_data);

  return (0);
}

int Construire_trame_ACK (
        TAddress AS,
        TAddress AD,
        Tampon T,
        unsigned short num,
        TTrame *Trame)
{
  Trame->AS = AS;
  Trame->AD = AD;

  Trame->TYPE = ACK;
  Trame->taille = 0;
  Trame->crc = 0;
  Trame->num = num;

  return (0);
}


int Envoyer_vers_couche_physique (TTrame Trame)
{
  char c;
  int fd;

  fd = open ("/dev/urandom", O_RDONLY);
  read (fd, &c, 1);
  close (fd);

  if (c % 3 || Trame.TYPE == ACK )
  {
    fd = open (FILE_PHY, O_RDWR | O_CREAT, 0600);

    write (fd, &Trame.AS, 1);
    write (fd, &Trame.AD, 1);
    write (fd, &Trame.TYPE, 1);
    printf("Envoit trame %i\n", Trame.num);
    write (fd, &Trame.num, 2); 
    write (fd, &Trame.taille, 1);
    if (Trame.taille)
      write (fd, Trame.ptr_data, Trame.taille);
    write (fd, &Trame.crc, 1);

    close (fd);
  }
  else
    fprintf (stderr, "Erreur de transmission: appel systeme casse ;)\n");
  return (0);
}

int Prendre_de_la_couche_physique (TTrame *Trame)
{
  int fd;
  fd = open (FILE_PHY, O_RDWR);

  read (fd, &Trame->AS, 1);
  read (fd, &Trame->AD, 1);
  read (fd, &Trame->TYPE, 1);
  read (fd, &Trame->num, 2);
  read (fd, &Trame->taille, 1);

  if (Trame->taille)
  {
    Trame->ptr_data = malloc (sizeof(unsigned char) * Trame->taille);
    read (fd, Trame->ptr_data, Trame->taille);
  }
  else
    Trame->ptr_data = NULL;

  read (fd, &Trame->crc, 1);

  close (fd);

  /* Vidage de la couche physique */
  printf("Effacage de la couche physique.\n");
  fd = open (FILE_PHY, O_RDWR | O_TRUNC);
  close (fd);

  return (0);
}

int Recuperer_donnees (
        TAddress *AS,
        TAddress *AD,
        Tampon *T,
        unsigned short *num,
        TTrame Trame)
{
  T->taille = Trame.taille;
  T->ptr_data = Trame.ptr_data;

  *AS = Trame.AS;
  *AD = Trame.AD;
  *num = Trame.num;
  
  return (0);
}

int Envoyer_vers_couche_reseau (
        Tampon phear,
        TAddress ASR,
        TAddress ADR)
{
  int fd;
  fd = open (FILE_NET_DST, O_RDWR | O_CREAT, 0600);

  write (fd, &ASR, 1);
  write (fd, &ADR, 1);
  write (fd, &(phear.taille), 1);
  write (fd, phear.ptr_data, phear.taille);

  fprintf (stderr, "Donnees recues: %s\n", phear.ptr_data);

  close (fd);
  return (0);
}

void vide_trame_tab (unsigned short NotACKTrame[NOMBRE_MEMOIRE])
{
  int i;

  for (i = 0; i < NOMBRE_MEMOIRE; i ++)
  {
    NotACKTrame[i] = 0;
  }

  return ;
}

int add_trame_tab (
    unsigned short NotACKTrame[NOMBRE_MEMOIRE],
    unsigned short num)
{
  int i;

  for (i = 0 ; i < NOMBRE_MEMOIRE; i ++)
  {
    if (NotACKTrame[i] == 0)
    {
      NotACKTrame[i] = num;
      return (i);
    }
  }

  return 0;
}

void rem_trame_tab (
    unsigned short NotACKTrame[NOMBRE_MEMOIRE],
    unsigned short num)
{
  int i;
  for (i = 0 ; i < NOMBRE_MEMOIRE; i ++)
  {
    if (NotACKTrame[i] == num)
      NotACKTrame[i] = 0;
  }

  return ;
}

int yatildelaplace_trame_tab (unsigned short NotACKTrame[NOMBRE_MEMOIRE])
{
  int i;

  for (i = 0 ; i < NOMBRE_MEMOIRE; i ++)
  {
    if (NotACKTrame[i] == 0)
      return 1;
  }
  return 0;
}

void copie_trame (TTrame *src, TTrame *dst)
{
  dst->AS        = src->AS;
  dst->AD        = src->AD;
  dst->TYPE      = src->TYPE;
  dst->num       = src->num;
  dst->taille    = src->taille;
  dst->ptr_data  = src->ptr_data;
  dst->crc       = src->crc;

  return ;
}
